from examples.functions_analytic_examples import *



#=======================================================
# REMOVE AND ADD COMMENTS TO RUN EXAMPLES
#=======================================================
def main():

    #run_example_charge_and_chargedensity()

    #run_example_setup_cycloidal_1qstates()

    #run_example_setup_domainwall_get_width()

    #run_example_setup_double_domainwall()

    #run_example_setup_skyrmion_get_radius()

    #run_example_setup_multilayer_hex()

    #run_example_setup_multilayer_quad()

    #run_example_setup_bimeron()

    #run_example_setup_nanoskyrmionlattice()

    #run_example_uudd_by_superposition()

    #run_example_setup_AFM_skyrmion()

    run_example_setup_chimera_state()


if __name__ == '__main__' :
     main()
     display_plots()
